from django.contrib.auth.decorators import login_required, permission_required
from django.urls import NoReverseMatch, reverse_lazy, reverse
from django.shortcuts import render
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.template.loader import get_template

from kmuhelper.utils import render_error
from kmuhelper.constants import URL_MANUAL

# Create your views here.


@login_required(login_url=reverse_lazy("admin:login"))
def home(request):
    grid = [
        [
            {
                "title": "Admin",
                "subtitle": "Volle Kontrolle über die Hauptfunktionen",
                "url": reverse("admin:app_list", kwargs={"app_label": "kmuhelper"}),
                "icon": "fas fa-key",
            },
            {
                "title": "E-Mails",
                "subtitle": "Verwaltung und Erstellung von E-Mails",
                "url": reverse("kmuhelper:email-index"),
                "icon": "fas fa-envelope-open",
            },
        ],
        [
            {
                "title": "App (Mobile)",
                "subtitle": "Eingeschränkte Verwaltung von Daten in verschiedenen Arbeitsschritten",
                "url": reverse("kmuhelper:app-mobile"),
                "icon": "fas fa-mobile-alt",
            },
            {
                "title": "App (Desktop)",
                "subtitle": "Eingeschränkte Verwaltung von Daten in verschiedenen Arbeitsschritten",
                "url": reverse("kmuhelper:app-desktop"),
                "icon": "fas fa-desktop",
            },
        ],
        [
            {
                "title": "Statistiken",
                "subtitle": "Diagramme und co.",
                "url": reverse("kmuhelper:stats"),
                "icon": "fas fa-chart-pie",
            },
            {
                "title": "Einstellungen",
                "subtitle": "Kleine Einstellungen für den KMUHelper",
                "url": reverse("admin:kmuhelper_einstellung_changelist"),
                "icon": "fas fa-cog",
            },
        ],
        [
            {
                "title": "Handbuch",
                "subtitle": "Dokumentation zum KMUHelper",
                "url": URL_MANUAL,
                "icon": "fas fa-book",
            },
        ]
    ]
    return render(request, "kmuhelper/home.html", {"has_permission": True, "grid": grid})


def error(request):
    return render_error(request)


@permission_required("is_superuser")
def _templatetest(request, templatename):
    """DEBUG: Test a template with given parameters"""

    try:
        get_template(templatename)
        return render(request, templatename, request.GET.dict())
    except TemplateDoesNotExist:
        return render_error(request, status=404, 
                            message=f"Vorlage '{templatename}' wurde nicht gefunden.")
    except TemplateSyntaxError as error:
        return render_error(request, status=400, 
                            message=f"Vorlage '{templatename}' enthält ungültige Syntax: {error}")
    except (NoReverseMatch) as error:
        return render_error(request, status=400, 
                            message=f"Er ist ein Fehler aufgetreten: {error}")



# Generated by Django 3.0.4 on 2020-04-28 15:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0005_auto_20200427_1857'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='bestellung',
            name='rechnungsnotiz',
        ),
        migrations.RemoveField(
            model_name='produkt',
            name='packlistenbemerkung',
        ),
        migrations.AlterField(
            model_name='produkt',
            name='bemerkung',
            field=models.TextField(blank=True, default='', help_text='Wird nicht gedruckt oder angezeigt; nur für eigene Zwecke.', verbose_name='Bemerkung'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-16 19:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0024_auto_20200716_1714'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ToDoNotiz',
            new_name='Notiz',
        ),
        migrations.CreateModel(
            name='ToDoNotiz',
            fields=[
            ],
            options={
                'verbose_name': 'Notiz',
                'verbose_name_plural': 'Notizen',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kmuhelper.notiz',),
        ),
        migrations.AlterModelOptions(
            name='notiz',
            options={'verbose_name': 'Notiz', 'verbose_name_plural': 'Notizen'},
        ),
        migrations.AlterModelOptions(
            name='todoversand',
            options={'verbose_name': 'Bestellung', 'verbose_name_plural': 'Bestellungen'},
        ),
        migrations.AlterModelOptions(
            name='todozahlungseingang',
            options={'verbose_name': 'Bestellung', 'verbose_name_plural': 'Bestellung'},
        ),
    ]



from datetime import datetime

from django.contrib.admin.models import LogEntry, CHANGE
from django.contrib.admin.options import get_content_type_for_model
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render
from django.urls import reverse_lazy, reverse

from django.utils import translation
_ = translation.gettext

from kmuhelper import constants
from kmuhelper.modules.main.models import Bestellung
from kmuhelper.decorators import require_object
from kmuhelper.utils import render_error
from kmuhelper.translations import Language

from kmuhelper.modules.pdfgeneration.order.forms import PDFOrderForm
from kmuhelper.modules.pdfgeneration.order.generator import PDFOrder

# Views

@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.view_bestellung")
@require_object(Bestellung)
def bestellung_pdf_ansehen(request, obj):
    order = obj
    is_print_version = 'print' in request.GET
    is_download = 'download' in request.GET

    preset = request.GET.get('preset', 'invoice')
    title = None
    text = None

    if 'custom' in request.GET:
        title = order.rechnungstitel
        text = order.rechnungstext

    # Get the language
    defaultlang = order.kunde.sprache if order.kunde and order.kunde.sprache else "de"
    lang = request.GET.get('language', defaultlang).lower()
    if lang not in dict(constants.LANGUAGES):
        lang = defaultlang

    with Language(lang):
        match preset:
            case 'invoice':
                title = title or _("Rechnung")
                pdf = PDFOrder(order, title, text=text, add_cut_lines=not is_print_version)
            case 'delivery-note':
                title = title or _("Lieferschein")
                pdf = PDFOrder(order, title, text=text, is_delivery_note=True)
            case 'payment-reminder':
                days_to_pay = 14
                title = title or _("Zahlungserinnerung")
                text = text or _("Geschätzter Kunde\n\nVermutlich ist Ihnen entgangen, diese Rechnung vom %(original_date)s innert der gewährten Frist zu begleichen.\nWir bitten Sie, dies innert %(days)d Tagen nachzuholen.\n\nSollte sich Ihre Zahlung mit diesem Schreiben überkreuzen, so betrachten Sie dieses bitte als gegenstandslos.") % {
                    'original_date': order.rechnungsdatum.strftime("%d.%m.%Y"),
                    'days': days_to_pay,
                }
                # Temporarily replace the order's date and payment conditions
                order.rechnungsdatum = datetime.now().date()
                order.zahlungskonditionen = f"0:{days_to_pay}"
                pdf = PDFOrder(order, title, text=text, add_cut_lines=not is_print_version, show_payment_conditions=False)
            case other:
                return render_error(request, status=400, message="Ungültige Vorlage: " + str(other))

        # Log the action
        LogEntry.objects.log_action(
            user_id=request.user.id, 
            content_type_id=get_content_type_for_model(obj).pk,
            object_id=obj.pk,
            object_repr=str(obj),
            action_flag=CHANGE,
            change_message=f'PDF "{title}" aus Vorlage "{preset}" erstellt mit Text: "{text}"' if text else f'PDF "{title}" aus Vorlage "{preset}" erstellt.',
        )

        filename = f'{str(order)} - {title}.pdf'
        return pdf.get_response(as_attachment=is_download, filename=filename)

@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.view_bestellung")
@require_object(Bestellung)
def bestellung_pdf_erstellen(request, obj):
    initial={
        'language': obj.kunde.sprache if obj.kunde and obj.kunde.sprache else 'de',
        'title': obj.rechnungstitel,
        'text': obj.rechnungstext,
    }
    
    if request.method == 'POST':
        form = PDFOrderForm(request.POST)
        if form.is_valid():
            form.update_order_settings(obj)
            url = reverse('admin:kmuhelper_bestellung_pdf', args=[obj.id])
            return redirect(url+form.get_url_params())
    else:
        form = PDFOrderForm(initial=initial)

    return render(request, 'admin/kmuhelper/bestellung/pdf_form.html', {
        'form': form,
        'has_permission': True,
        'original': obj,
    })



from django.contrib import admin
from django.urls import path

from kmuhelper.modules.emails import views
from kmuhelper.modules.emails.models import EMail, EMailAttachment, Attachment, EMailTemplate
from kmuhelper.overrides import CustomModelAdmin, CustomTabularInline

#######


@admin.register(Attachment)
class AttachmentAdmin(CustomModelAdmin):
    list_display = ['filename', 'description',
                    'time_created', 'autocreated', 'id']
    list_display_links = ['filename']

    search_fields = ('filename', 'description', )

    ordering = ['-time_created']

    save_on_top = True

    hidden = True

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ['time_created', 'file', 'autocreated']

        return ['time_created', 'autocreated']

    def get_fieldsets(self, request, obj=None):
        default = [
            ('Datei', {'fields': ['filename', 'file']}),
            ('Infos', {'fields': ['description']})
        ]

        if obj:
            return default + [
                ('Daten', {'fields': ['autocreated', 'time_created']}),
            ]

        return default

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/download/', self.admin_site.admin_view(views.attachment_download),
                 name='%s_%s_download' % info),
        ]
        return my_urls + urls


class EMailAdminAttachmentInline(CustomTabularInline):
    model = EMailAttachment
    verbose_name = "Anhang"
    verbose_name_plural = "Anhänge"
    extra = 0

    show_change_link = True

    autocomplete_fields = ('attachment', )

    # Permissions

    NO_CHANGE = True


@admin.register(EMail)
class EMailAdmin(CustomModelAdmin):
    def get_fieldsets(self, request, obj=None):
        default = [
            ('Infos', {
                'fields': ['subject', 'to']}),
            ('Zusätzliche Empfänger', {
                'fields': ['cc', 'bcc'],
                'classes': ['collapse']}),
            ('Inhalt', {
                'fields': ['html_template', 'text', 'html_context']}),
        ]

        if obj:
            return default + [
                ('Zeiten', {'fields': ['time_created', 'time_sent']}),
            ]

        return default

    readonly_fields = ('time_created', 'time_sent')

    ordering = ('sent', '-time_sent', '-time_created')

    list_display = ('subject', 'to', 'html_template',
                    'time_created', 'sent', 'time_sent')
    list_filter = ('html_template', 'sent')

    search_fields = ['subject', 'to', 'cc',
                     'bcc', 'html_context', 'text', 'notes']

    inlines = (EMailAdminAttachmentInline, )

    save_on_top = True

    hidden = True

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/preview/', self.admin_site.admin_view(views.email_preview),
                 name='%s_%s_preview' % info),
            path('<path:object_id>/send/', self.admin_site.admin_view(views.email_send),
                 name='%s_%s_send' % info),
            path('<path:object_id>/resend/', self.admin_site.admin_view(views.email_resend),
                 name='%s_%s_resend' % info),
        ]
        return my_urls + urls

    # Custom save

    def save_model(self, request, obj, form, change):
        if obj:
            obj.is_valid(request)
        super().save_model(request, obj, form, change)


@admin.register(EMailTemplate)
class EMailTemplateAdmin(CustomModelAdmin):
    list_display = ['title', 'description', 'get_use_link']

    search_fields = ('title', 'description', )

    ordering = ['title']

    fieldsets = [
        ('Infos', {
            'fields': ['title', 'description']}),
        ('Inhalt', {
            'fields': ['mail_to', 'mail_subject', 'mail_text']}),
        ('Optionen', {
            'fields': ['mail_template', 'mail_context']}),
    ]

    save_on_top = True

    hidden = True

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('savevars', self.admin_site.admin_view(views.emailtemplate_savevars),
                 name="%s_%s_savevars" % info),
            path('resetvars', self.admin_site.admin_view(views.emailtemplate_resetvars),
                 name="%s_%s_resetvars" % info),
            path('<path:object_id>/use/', self.admin_site.admin_view(views.emailtemplate_use),
                 name='%s_%s_use' % info),
        ]
        return my_urls + urls

#


modeladmins = [
    (EMail, EMailAdmin),
    (Attachment, AttachmentAdmin),
    (EMailTemplate, EMailTemplateAdmin),
]



import datetime
import json

from django.contrib.auth.decorators import login_required, permission_required
from django.core.serializers.json import DjangoJSONEncoder
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils import timezone

from kmuhelper.utils import render_error
from kmuhelper.translations import langselect
from kmuhelper.modules.main.models import Bestellungsposten, Bestellung


#####


@login_required(login_url=reverse_lazy("admin:login"))
def stats(request):
    return render(request, "kmuhelper/stats/index.html", {"has_permission": True})


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.view_produkt")
def stats_products_price(request):
    if not Bestellung.objects.exists():
        return render_error(request, status=400,
                            message="Keine Bestellungen vorhanden.")

    try:
        from_date = timezone.make_aware(datetime.datetime.strptime(
            str(request.GET.get("from")), "%Y-%m-%d"))
    except (ValueError, IndexError):
        from_date = Bestellung.objects.order_by("datum").first().datum

    try:
        to_date = timezone.make_aware(datetime.datetime.strptime(
            str(request.GET.get("to")), "%Y-%m-%d"))
    except (ValueError, IndexError):
        to_date = Bestellung.objects.order_by("datum").last().datum

    products_sold = {}

    for bp in Bestellungsposten.objects.filter(bestellung__datum__gte=from_date, bestellung__datum__lte=to_date).order_by("bestellung__datum").values("bestellung__datum", "menge"):
        d = datetime.date(year=bp["bestellung__datum"].year,
                          month=bp["bestellung__datum"].month, day=1).isoformat()
        if d in products_sold:
            products_sold[d] += bp["menge"]
        else:
            products_sold[d] = bp["menge"]

    products_sold = [{"date": date, "y": products_sold[date]}
                     for date in products_sold]
    products_sold_data = json.dumps(products_sold, cls=DjangoJSONEncoder)

    money_income = {}

    for b in Bestellung.objects.filter(datum__gte=from_date, datum__lte=to_date).order_by("datum").values("datum", "fix_summe"):
        d = datetime.date(year=b["datum"].year,
                          month=b["datum"].month, day=1).isoformat()
        if d in money_income:
            money_income[d] += b["fix_summe"]
        else:
            money_income[d] = b["fix_summe"]

    money_income = [{"date": date, "y": money_income[date]}
                    for date in money_income]
    money_income_data = json.dumps(money_income, cls=DjangoJSONEncoder)

    context = {
        "has_permission": True,
        "products_sold": products_sold_data,
        "money_income": money_income_data,
    }
    return render(request, "kmuhelper/stats/products_price.html", context)


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.view_produkt")
def best_products(request):
    if not Bestellung.objects.exists():
        return render_error(request, status=400,
                            message="Keine Bestellungen vorhanden.")

    try:
        if "max" in request.GET:
            max_count = int(request.GET.get("max"))
        else:
            max_count = 20
    except (ValueError, IndexError):
        max_count = 20

    try:
        from_date = timezone.make_aware(datetime.datetime.strptime(
            str(request.GET.get("from")), "%Y-%m-%d"))
    except (ValueError, IndexError):
        from_date = Bestellung.objects.order_by("datum").first().datum

    try:
        to_date=timezone.make_aware(datetime.datetime.strptime(
            str(request.GET.get("to")), "%Y-%m-%d"))
    except (ValueError, IndexError):
        to_date = Bestellung.objects.order_by("datum").last().datum

    products = {}

    for bp in Bestellungsposten.objects.filter(bestellung__datum__gte=from_date, bestellung__datum__lte=to_date).order_by("produkt__name").values("produkt__name", "menge"):
        if bp["produkt__name"] in products:
            products[langselect(bp["produkt__name"])] += bp["menge"]
        else:
            products[langselect(bp["produkt__name"])] = bp["menge"]

    products = sorted(products.items(), key=lambda x: x[1], reverse=True)
    products = products[:max_count] if len(products) > max_count else products

    context = {
        "has_permission": True,
        "product_names": [p[0] for p in products],
        "product_counts": [p[1] for p in products],
    }
    return render(request, "kmuhelper/stats/best_products.html", context)



# Generated by Django 3.1.7 on 2021-03-24 19:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('kmuhelper', '0050_produkt_lieferant_url'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApiKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.UUIDField(default=uuid.uuid4, unique=True, verbose_name='Key')),
                ('name', models.CharField(blank=True, default='', max_length=100, verbose_name='Name')),
                ('read', models.BooleanField(default=True, verbose_name='Read permission?')),
                ('write', models.BooleanField(default=False, verbose_name='Write permission?')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Api key',
                'verbose_name_plural': 'Api keys',
            },
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-15 17:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0018_auto_20200715_1639'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='kunde',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Kunde'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-17 12:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0025_auto_20200716_2126'),
    ]

    operations = [
        migrations.CreateModel(
            name='ToDoLagerbestand',
            fields=[
            ],
            options={
                'verbose_name': 'Produkt',
                'verbose_name_plural': 'Produkte',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kmuhelper.produkt',),
        ),
        migrations.CreateModel(
            name='ToDoLieferung',
            fields=[
            ],
            options={
                'verbose_name': 'Lieferung',
                'verbose_name_plural': 'Lieferungen',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kmuhelper.lieferung',),
        ),
    ]



from django.urls import path

from kmuhelper.modules.integrations.woocommerce import views

#######################

urlpatterns = [
    path('auth/start',
         views.wc_auth_start, name="wc-auth-start"),
    path('auth/end',
         views.wc_auth_end, name="wc-auth-end"),
    path('auth/key',
         views.wc_auth_key, name="wc-auth-key"),

    path('import/products',
         views.wc_import_products, name="wc-import-products"),
    path('import/customers',
         views.wc_import_customers, name="wc-import-customers"),
    path('import/categories',
         views.wc_import_categories, name="wc-import-categories"),
    path('import/orders',
         views.wc_import_orders, name="wc-import-orders"),

    path('update/product/<int:object_id>',
         views.wc_update_product, name="wc-update-product"),
    path('update/customer/<int:object_id>',
         views.wc_update_customer, name="wc-update-customer"),
    path('update/category/<int:object_id>',
         views.wc_update_category, name="wc-update-category"),
    path('update/order/<int:object_id>',
         views.wc_update_order, name="wc-update-order"),

    path('webhooks', views.wc_webhooks, name="wc-webhooks"),
]



# Generated by Django 3.0.4 on 2020-08-01 22:18

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0031_auto_20200727_1603'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='datum',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Datum'),
        ),
    ]



from django.contrib import admin, messages
from django.urls import path

from kmuhelper.overrides import CustomModelAdmin
from kmuhelper.modules.integrations.paymentimport.models import PaymentImport, PaymentImportEntry
from kmuhelper.modules.integrations.paymentimport import views


class PaymentImportAdminEntryInline(admin.TabularInline):
    model = PaymentImportEntry
    verbose_name = "Eintrag"
    verbose_name_plural = "Einträge"
    extra = 0

    fields = ('currency', 'betrag', 'ref', 'order_id',
              'valuedate', 'name', 'iban', 'additionalref')
    readonly_fields = ('order_id', 'betrag',)
    ordering = ('ref',)


@admin.register(PaymentImport)
class PaymentImportAdmin(CustomModelAdmin):
    readonly_fields = ('time_imported',)

    fieldsets = [
        ('Infos', {'fields': ['is_processed', 'time_imported']}),
        ('Daten', {'fields': ['data_msgid', 'data_creationdate']}),
    ]

    ordering = ('is_processed',)

    list_display = ('time_imported', 'entrycount', 'is_processed', 'data_msgid',)
    list_filter = ('is_processed',)

    inlines = (PaymentImportAdminEntryInline, )

    save_on_top = True
    hidden = True

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('upload/', self.admin_site.admin_view(views.upload),
                 name="%s_%s_upload" % info),
            path('<path:object_id>/process/', self.admin_site.admin_view(views.process),
                 name="%s_%s_process" % info),
        ]
        return my_urls + urls

    # Permissions

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return obj is None

#


modeladmins = [
    (PaymentImport, PaymentImportAdmin),
]
